@@ -1,4 +1,4 @@
-/* $Id: time.cpp 122588 2018-05-08 18:08:52Z klaus $ */
+/* $Id: time.cpp 128120 2019-01-11 19:34:57Z bird $ */
 /** @file
  * IPRT - Time.
  */
@@ -236,6 +236,17 @@ int main()
 }
 */
 
+/** RFC-1123 week day names. */
+static const char * const g_apszWeekDays[7] =
+{
+    "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"
+};
+/** RFC-1123 month of the year names. */
+static const char * const g_apszMonths[1+12] =
+{
+    "000", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+};
+
 
 /**
  * Checks if a year is a leap year or not.
@@ -787,6 +798,74 @@ RTDECL(char *) RTTimeToString(PCRTTIME pTime, char *psz, size_t cb)
 RT_EXPORT_SYMBOL(RTTimeToString);
 
 
+/**
+ * Converts a time spec to a ISO date string, extended version.
+ *
+ * @returns Output string length on success (positive), VERR_BUFFER_OVERFLOW
+ *          (negative) or VERR_OUT_OF_RANGE (negative) on failure.
+ * @param   pTime           The time. Caller should've normalized this.
+ * @param   psz             Where to store the string.
+ * @param   cb              The size of the buffer.
+ * @param   cFractionDigits Number of digits in the fraction.  Max is 9.
+ */
+RTDECL(ssize_t) RTTimeToStringEx(PCRTTIME pTime, char *psz, size_t cb, unsigned cFractionDigits)
+{
+    size_t cch;
+
+    /* Format the fraction. */
+    char szFraction[16];
+    if (!cFractionDigits)
+        szFraction[0] = '\0';
+    else
+    {
+        AssertReturn(cFractionDigits <= 9, VERR_OUT_OF_RANGE);
+        Assert(pTime->u32Nanosecond <= 999999999);
+        RTStrPrintf(szFraction, sizeof(szFraction), ".%09RU32", pTime->u32Nanosecond);
+        szFraction[cFractionDigits + 1] = '\0';
+    }
+
+    /* (Default to UTC if not specified) */
+    if (    (pTime->fFlags & RTTIME_FLAGS_TYPE_MASK) == RTTIME_FLAGS_TYPE_LOCAL
+        &&  pTime->offUTC)
+    {
+        int32_t  offUTC = pTime->offUTC;
+        Assert(offUTC <= 840 && offUTC >= -840);
+        char     chSign;
+        if (offUTC >= 0)
+            chSign = '+';
+        else
+        {
+            chSign = '-';
+            offUTC = -offUTC;
+        }
+        uint32_t offUTCHour   = (uint32_t)offUTC / 60;
+        uint32_t offUTCMinute = (uint32_t)offUTC % 60;
+
+        /* Examples: 2018-09-07T16:12:00+02:00  2018-09-07T16:12:00.123456789+02:00 */
+        cch = RTStrPrintf(psz, cb,
+                          "%04RI32-%02u-%02uT%02u:%02u:%02u%s%c%02d%:02d",
+                          pTime->i32Year, pTime->u8Month, pTime->u8MonthDay,
+                          pTime->u8Hour, pTime->u8Minute, pTime->u8Second, szFraction,
+                          chSign, offUTCHour, offUTCMinute);
+        if (   cch >= 24
+            && psz[cch - 6] == chSign)
+            return cch;
+    }
+    else
+    {
+        /* Examples: 2018-09-07T16:12:00Z  2018-09-07T16:12:00.123456789Z */
+        cch = RTStrPrintf(psz, cb, "%04RI32-%02u-%02uT%02u:%02u:%02u%sZ",
+                          pTime->i32Year, pTime->u8Month, pTime->u8MonthDay,
+                          pTime->u8Hour, pTime->u8Minute, pTime->u8Second, szFraction);
+        if (   cch >= 19
+            && psz[cch - 1] == 'Z')
+            return cch;
+    }
+    return VERR_BUFFER_OVERFLOW;
+}
+RT_EXPORT_SYMBOL(RTTimeToStringEx);
+
+
 /**
  * Converts a time spec to a ISO date string.
  *
@@ -829,7 +908,7 @@ RTDECL(PRTTIME) RTTimeFromString(PRTTIME pTime, const char *pszString)
     pTime->offUTC = 0;
 
     /*
-     * The day part.
+     * The date part.
      */
 
     /* Year */
@@ -900,14 +979,31 @@ RTDECL(PRTTIME) RTTimeFromString(PRTTIME pTime, const char *pszString)
     if (pTime->u8Second > 59)
         return NULL;
 
-    /* Nanoseconds is optional and probably non-standard. */
+    /* Just in case there is a fraction of seconds (should be!). */
     if (*pszString == '.')
     {
+        const char * const pszStart = pszString;
         rc = RTStrToUInt32Ex(pszString + 1, (char **)&pszString, 10, &pTime->u32Nanosecond);
         if (rc != VINF_SUCCESS && rc != VWRN_TRAILING_CHARS && rc != VWRN_TRAILING_SPACES)
             return NULL;
         if (pTime->u32Nanosecond >= 1000000000)
             return NULL;
+        switch (pszString - pszStart)
+        {
+            case 1: pTime->u32Nanosecond *= 100000000; break;
+            case 2: pTime->u32Nanosecond *= 10000000; break;
+            case 3: pTime->u32Nanosecond *= 1000000; break;
+            case 4: pTime->u32Nanosecond *= 100000; break;
+            case 5: pTime->u32Nanosecond *= 10000; break;
+            case 6: pTime->u32Nanosecond *= 1000; break;
+            case 7: pTime->u32Nanosecond *= 100; break;
+            case 8: pTime->u32Nanosecond *= 10; break;
+            case 9: break;
+            default:
+                return NULL;
+        }
+        if (pTime->u32Nanosecond >= 1000000000)
+            return NULL;
     }
     else
         pTime->u32Nanosecond = 0;
@@ -981,6 +1077,307 @@ RTDECL(PRTTIMESPEC) RTTimeSpecFromString(PRTTIMESPEC pTime, const char *pszStrin
 RT_EXPORT_SYMBOL(RTTimeSpecFromString);
 
 
+/**
+ * Formats the given time on a RTC-2822 compliant format.
+ *
+ * @returns Output string length on success (positive), VERR_BUFFER_OVERFLOW
+ *          (negative) on failure.
+ * @param   pTime       The time. Caller should've normalized this.
+ * @param   psz         Where to store the string.
+ * @param   cb          The size of the buffer.
+ */
+RTDECL(ssize_t) RTTimeToRfc2822(PRTTIME pTime, char *psz, size_t cb, uint32_t fFlags)
+{
+    Assert(pTime->u8Month > 0 && pTime->u8Month <= 12);
+    Assert(pTime->u8WeekDay < 7);
+    Assert(!(fFlags & ~RTTIME_RFC2822_F_GMT));
+
+    /* (Default to UTC if not specified) */
+    if (   (pTime->fFlags & RTTIME_FLAGS_TYPE_MASK) == RTTIME_FLAGS_TYPE_LOCAL
+        && pTime->offUTC)
+    {
+        Assert(!(fFlags & RTTIME_RFC2822_F_GMT) /* don't call with local time. duh! */ );
+
+        /* Calc the UTC offset part. */
+        int32_t offUtc = pTime->offUTC;
+        Assert(offUtc <= 840 && offUtc >= -840);
+        char     chSign;
+        if (offUtc >= 0)
+            chSign = '+';
+        else
+        {
+            chSign = '-';
+            offUtc = -offUtc;
+        }
+        uint32_t offUtcHour   = (uint32_t)offUtc / 60;
+        uint32_t offUtcMinute = (uint32_t)offUtc % 60;
+
+        /* Example:                       "Mon, 31 Aug 2018 00:00:00 +0200" */
+        size_t cch = RTStrPrintf(psz, cb, "%s, %u %s %04RI32 %02u:%02u:%02u %c%02u%02u", g_apszWeekDays[pTime->u8WeekDay],
+                                 pTime->u8MonthDay, g_apszMonths[pTime->u8Month], pTime->i32Year,
+                                 pTime->u8Hour, pTime->u8Minute, pTime->u8Second, chSign, offUtcHour, offUtcMinute);
+        if (   cch >= 27
+            && psz[cch - 5] == chSign)
+            return cch;
+    }
+    else if (fFlags & RTTIME_RFC2822_F_GMT)
+    {
+        /* Example:                       "Mon, 1 Jan 1971 23:55:59 GMT" */
+        size_t cch = RTStrPrintf(psz, cb, "%s, %u %s %04RI32 %02u:%02u:%02u GMT", g_apszWeekDays[pTime->u8WeekDay],
+                                 pTime->u8MonthDay, g_apszMonths[pTime->u8Month], pTime->i32Year,
+                                 pTime->u8Hour, pTime->u8Minute, pTime->u8Second);
+        if (   cch >= 27
+            && psz[cch - 1] == 'T')
+            return cch;
+    }
+    else
+    {
+        /* Example:                       "Mon, 1 Jan 1971 00:00:00 -0000" */
+        size_t cch = RTStrPrintf(psz, cb, "%s, %u %s %04RI32 %02u:%02u:%02u -0000", g_apszWeekDays[pTime->u8WeekDay],
+                                 pTime->u8MonthDay, g_apszMonths[pTime->u8Month], pTime->i32Year,
+                                 pTime->u8Hour, pTime->u8Minute, pTime->u8Second);
+        if (   cch >= 27
+            && psz[cch - 5] == '-')
+            return cch;
+    }
+    return VERR_BUFFER_OVERFLOW;
+}
+RT_EXPORT_SYMBOL(RTTimeToRfc2822);
+
+
+/**
+ * Attempts to convert an RFC-2822 date string to a time structure.
+ *
+ * We're a little forgiving with zero padding, unspecified parts, and leading
+ * and trailing spaces.
+ *
+ * @retval  pTime on success,
+ * @retval  NULL on failure.
+ * @param   pTime       Where to store the time on success.
+ * @param   pszString   The ISO date string to convert.
+ */
+RTDECL(PRTTIME) RTTimeFromRfc2822(PRTTIME pTime, const char *pszString)
+{
+    /*
+     * Fri, 31 Aug 2018 00:00:00 +0200
+     * Mon, 3 Sep 2018 00:00:00 GMT
+     * Mon, 3 Sep 2018 00:00:00 -0000
+     * 3 Sep 2018 00:00:00 -0000 (?)
+     * 3 Sep 2018 00:00:00 GMT   (?)
+     *
+     */
+
+    /* Ignore leading spaces. */
+    while (RT_C_IS_SPACE(*pszString))
+        pszString++;
+
+    /*
+     * Init non date & time parts.
+     */
+    pTime->fFlags = RTTIME_FLAGS_TYPE_LOCAL;
+    pTime->offUTC = 0;
+
+    /*
+     * The date part.
+     */
+
+    /* Optional day of week: */
+    if (RT_C_IS_ALPHA(pszString[0]))
+        pTime->u8WeekDay = UINT8_MAX;
+    else if (pszString[0] != '\0' && pszString[1] != '\0')
+    {
+        uint32_t uWeekDay = RT_MAKE_U32_FROM_U8(RT_C_TO_LOWER(pszString[0]), RT_C_TO_LOWER(pszString[0]),
+                                                RT_C_TO_LOWER(pszString[1]), 0);
+        if (     uWeekDay == RT_MAKE_U32_FROM_U8('m', 'o', 'n', 0))     pTime->u8WeekDay = 0;
+        else if (uWeekDay == RT_MAKE_U32_FROM_U8('t', 'u', 'e', 0))     pTime->u8WeekDay = 1;
+        else if (uWeekDay == RT_MAKE_U32_FROM_U8('w', 'e', 'd', 0))     pTime->u8WeekDay = 2;
+        else if (uWeekDay == RT_MAKE_U32_FROM_U8('t', 'h', 'u', 0))     pTime->u8WeekDay = 3;
+        else if (uWeekDay == RT_MAKE_U32_FROM_U8('f', 'r', 'i', 0))     pTime->u8WeekDay = 4;
+        else if (uWeekDay == RT_MAKE_U32_FROM_U8('s', 'a', 't', 0))     pTime->u8WeekDay = 5;
+        else if (uWeekDay == RT_MAKE_U32_FROM_U8('s', 'u', 'n', 0))     pTime->u8WeekDay = 6;
+        else
+            return NULL;
+        pszString += 3;
+        while (RT_C_IS_ALPHA(*pszString))
+            pszString++;
+        if (*pszString == ',')
+            pszString++;
+        while (RT_C_IS_SPACE(*pszString))
+            pszString++;
+        if (!RT_C_IS_DIGIT(pszString[0]))
+            return NULL;
+    }
+    else
+        return NULL;
+
+    /* Day of month.*/
+    int rc = RTStrToUInt8Ex(pszString, (char **)&pszString, 10, &pTime->u8MonthDay);
+    if (rc != VWRN_TRAILING_CHARS && rc != VINF_SUCCESS)
+        return NULL;
+    while (RT_C_IS_SPACE(*pszString))
+        pszString++;
+
+    /* Month of the year. */
+    if (pszString[0] == '\0' || pszString[1] == '\0' || pszString[2] == '\0')
+        return NULL;
+    uint32_t uMonth = RT_MAKE_U32_FROM_U8(RT_C_TO_LOWER(pszString[0]), RT_C_TO_LOWER(pszString[0]),
+                                          RT_C_TO_LOWER(pszString[1]), 0);
+    if (     uMonth == RT_MAKE_U32_FROM_U8('j', 'a', 'n', 0))     pTime->u8Month = 1;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('f', 'e', 'b', 0))     pTime->u8Month = 2;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('m', 'a', 'r', 0))     pTime->u8Month = 3;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('a', 'p', 'r', 0))     pTime->u8Month = 4;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('m', 'a', 'y', 0))     pTime->u8Month = 5;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('j', 'u', 'n', 0))     pTime->u8Month = 6;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('j', 'u', 'l', 0))     pTime->u8Month = 7;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('a', 'u', 'g', 0))     pTime->u8Month = 8;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('s', 'e', 'p', 0))     pTime->u8Month = 9;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('o', 'c', 't', 0))     pTime->u8Month = 10;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('n', 'o', 'v', 0))     pTime->u8Month = 11;
+    else if (uMonth == RT_MAKE_U32_FROM_U8('d', 'e', 'c', 0))     pTime->u8Month = 12;
+    else
+        return NULL;
+    pszString += 3;
+    while (RT_C_IS_ALPHA(*pszString))
+        pszString++;
+    while (RT_C_IS_SPACE(*pszString))
+        pszString++;
+
+    /* Year */
+    rc = RTStrToInt32Ex(pszString, (char **)&pszString, 10, &pTime->i32Year);
+    if (rc != VWRN_TRAILING_CHARS)
+        return NULL;
+
+    bool const fLeapYear = rtTimeIsLeapYear(pTime->i32Year);
+    if (fLeapYear)
+        pTime->fFlags |= RTTIME_FLAGS_LEAP_YEAR;
+
+    while (RT_C_IS_SPACE(*pszString))
+        pszString++;
+
+
+    /* Calculate year day. */
+    unsigned const cDaysInMonth = fLeapYear
+                                ? g_acDaysInMonthsLeap[pTime->u8Month - 1]
+                                : g_acDaysInMonths[pTime->u8Month - 1];
+    if (pTime->u8MonthDay == 0 || pTime->u8MonthDay > cDaysInMonth)
+        return NULL;
+
+    pTime->u16YearDay = pTime->u8MonthDay - 1
+                      + (fLeapYear
+                         ? g_aiDayOfYearLeap[pTime->u8Month - 1]
+                         : g_aiDayOfYear[pTime->u8Month - 1]);
+
+    /*
+     * The time part.
+     */
+    /* Hour. */
+    rc = RTStrToUInt8Ex(pszString, (char **)&pszString, 10, &pTime->u8Hour);
+    if (rc != VWRN_TRAILING_CHARS)
+        return NULL;
+    if (pTime->u8Hour > 23)
+        return NULL;
+    if (*pszString++ != ':')
+        return NULL;
+
+    /* Minute. */
+    rc = RTStrToUInt8Ex(pszString, (char **)&pszString, 10, &pTime->u8Minute);
+    if (rc != VWRN_TRAILING_CHARS)
+        return NULL;
+    if (pTime->u8Minute > 59)
+        return NULL;
+    if (*pszString++ != ':')
+        return NULL;
+
+    /* Second. */
+    rc = RTStrToUInt8Ex(pszString, (char **)&pszString, 10, &pTime->u8Second);
+    if (rc != VINF_SUCCESS && rc != VWRN_TRAILING_CHARS && rc != VWRN_TRAILING_SPACES)
+        return NULL;
+    if (pTime->u8Second > 59)
+        return NULL;
+
+    /* Nanoseconds  and probably non-standard. */
+    if (*pszString == '.')
+    {
+        const char * const pszStart = pszString;
+        rc = RTStrToUInt32Ex(pszString + 1, (char **)&pszString, 10, &pTime->u32Nanosecond);
+        if (rc != VINF_SUCCESS && rc != VWRN_TRAILING_CHARS && rc != VWRN_TRAILING_SPACES)
+            return NULL;
+        if (pTime->u32Nanosecond >= 1000000000)
+            return NULL;
+        switch (pszString - pszStart)
+        {
+            case 1: pTime->u32Nanosecond *= 100000000; break;
+            case 2: pTime->u32Nanosecond *= 10000000; break;
+            case 3: pTime->u32Nanosecond *= 1000000; break;
+            case 4: pTime->u32Nanosecond *= 100000; break;
+            case 5: pTime->u32Nanosecond *= 10000; break;
+            case 6: pTime->u32Nanosecond *= 1000; break;
+            case 7: pTime->u32Nanosecond *= 100; break;
+            case 8: pTime->u32Nanosecond *= 10; break;
+            case 9: break;
+            default:
+                return NULL;
+        }
+        if (pTime->u32Nanosecond >= 1000000000)
+            return NULL;
+    }
+    else
+        pTime->u32Nanosecond = 0;
+
+    /*
+     * Time zone.
+     */
+    if (   (pszString[0] == 'G' || pszString[0] == 'g')
+        && (pszString[1] == 'M' || pszString[1] == 'm')
+        && (pszString[2] == 'T' || pszString[2] == 't') )
+    {
+        pszString++;
+        pTime->fFlags &= ~RTTIME_FLAGS_TYPE_MASK;
+        pTime->fFlags |= ~RTTIME_FLAGS_TYPE_UTC;
+        pTime->offUTC = 0;
+    }
+    else if (   *pszString == '+'
+             || *pszString == '-')
+    {
+        if (   !RT_C_IS_DIGIT(pszString[1])
+            || !RT_C_IS_DIGIT(pszString[2]))
+            return NULL;
+        int8_t cUtcHours = (pszString[1] - '0') * 10 + (pszString[2] - '0');
+        if (*pszString == '-')
+            cUtcHours = -cUtcHours;
+        pszString += 3;
+
+        uint8_t cUtcMin = 0;
+        if (RT_C_IS_DIGIT(pszString[0]))
+        {
+            rc = RTStrToUInt8Ex(pszString, (char **)&pszString, 10, &cUtcMin);
+            if (rc != VINF_SUCCESS && rc != VWRN_TRAILING_SPACES)
+                return NULL;
+        }
+        else if (*pszString && !RT_C_IS_BLANK(*pszString))
+            return NULL;
+        if (cUtcHours >= 0)
+            pTime->offUTC = cUtcHours * 60 + cUtcMin;
+        else
+            pTime->offUTC = cUtcHours * 60 - cUtcMin;
+        if (RT_ABS(pTime->offUTC) > 840)
+            return NULL;
+    }
+    /* else: No time zone given, local with offUTC = 0. */
+
+    /*
+     * The rest of the string should be blanks.
+     */
+    char ch;
+    while ((ch = *pszString++) != '\0')
+        if (!RT_C_IS_BLANK(ch))
+            return NULL;
+
+    return pTime;
+}
+RT_EXPORT_SYMBOL(RTTimeFromRfc2822);
+
+
 /**
  * Adds one day to @a pTime.
  *
