@@ -1,4 +1,4 @@
-/* $Id: thread2-r0drv-linux.c 118412 2017-10-17 14:26:02Z bird $ */
+/* $Id: thread2-r0drv-linux.c 134482 2019-11-06 17:42:39Z fbatschu $ */
 /** @file
  * IPRT - Threads (Part 2), Ring-0 Driver, Linux.
  */
@@ -36,6 +36,9 @@
 #include <iprt/err.h>
 #include "internal/thread.h"
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 11, 0)
+    #include <uapi/linux/sched/types.h>
+#endif /* >= KERNEL_VERSION(4, 11, 0) */
 
 RTDECL(RTTHREAD) RTThreadSelf(void)
 {
