@@ -1,10 +1,10 @@
-/* $Id: threadctxhooks-r0drv-linux.c 102121 2015-08-14 15:16:38Z bird $ */
+/* $Id: threadctxhooks-r0drv-linux.c 118412 2017-10-17 14:26:02Z bird $ */
 /** @file
  * IPRT - Thread Context Switching Hook, Ring-0 Driver, Linux.
  */
 
 /*
- * Copyright (C) 2013-2015 Oracle Corporation
+ * Copyright (C) 2013-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -98,6 +98,7 @@ static void rtThreadCtxHooksLnxSchedOut(struct preempt_notifier *pPreemptNotifie
     RTCCUINTREG fSavedEFlags = ASMGetFlags();
     stac();
 #endif
+    RT_NOREF_PV(pNext);
 
     AssertPtr(pThis);
     AssertPtr(pThis->pfnCallback);
@@ -132,6 +133,7 @@ static void rtThreadCtxHooksLnxSchedIn(struct preempt_notifier *pPreemptNotifier
     RTCCUINTREG fSavedEFlags = ASMGetFlags();
     stac();
 #endif
+    RT_NOREF_PV(iCpu);
 
     AssertPtr(pThis);
     AssertPtr(pThis->pfnCallback);
